@@ -9,8 +9,8 @@
 #
 package Test::Moose::More;
 our $AUTHORITY = 'cpan:RSRCHBOY';
-# git description: 0.037-3-g7fcead6
-$Test::Moose::More::VERSION = '0.038';
+# git description: 0.041-8-gc7df3d5
+$Test::Moose::More::VERSION = '0.042';
 
 # ABSTRACT: More tools for testing Moose packages
 
@@ -48,8 +48,14 @@ use Sub::Exporter::Progressive -setup => {
         is_class
         is_not_anon
         is_role
+
+        does_metaroles_ok
+        does_not_metaroles_ok
     } ],
-    groups  => { default => [ ':all' ] },
+    groups => {
+        default  => [ ':all' ],
+        validate => [ map { "validate_$_" } qw{ attribute class role thing } ],
+    },
 };
 
 use Test::Builder;
@@ -61,6 +67,8 @@ use Moose::Util 'resolve_metatrait_alias', 'does_role', 'find_meta';
 use Moose::Util::TypeConstraints;
 use Data::OptList;
 
+use Test::Moose::More::Utils;
+
 # debugging...
 #use Smart::Comments;
 
@@ -97,7 +105,7 @@ sub meta_ok ($;$) {
 }
 
 
-sub does_ok {
+sub does_ok ($$;$) {
     my ($thing, $roles, $message) = @_;
 
     my $thing_meta = find_meta($thing);
@@ -105,15 +113,22 @@ sub does_ok {
     $roles     = [ $roles ] unless ref $roles;
     $message ||= _thing_name($thing, $thing_meta) . ' does %s';
 
+    # this generally happens when we're checking a vanilla attribute
+    # metaclass, which turns out to be a
+    # Class::MOP::Class::Immutable::Class::MOP::Class.  If our metaclass does
+    # not have a does_role() method, then by definition the metaclass cannot
+    # do the role (that is, it's a Class::MOP metaclass).
+    my $_does = $thing_meta->can('does_role') || sub { 0 };
+
     BEGIN { warnings::unimport 'redundant' if $^V gt v5.21.1 }
-    $tb->ok(!!$thing_meta->does_role($_), sprintf($message, $_))
+    $tb->ok(!!$thing_meta->$_does($_), sprintf($message, $_))
         for @$roles;
 
     return;
 }
 
 
-sub does_not_ok {
+sub does_not_ok ($$;$) {
     my ($thing, $roles, $message) = @_;
 
     my $thing_meta = find_meta($thing);
@@ -121,8 +136,10 @@ sub does_not_ok {
     $roles     = [ $roles ] unless ref $roles;
     $message ||= _thing_name($thing, $thing_meta) . ' does not do %s';
 
+    my $_does = $thing_meta->can('does_role') || sub { 0 };
+
     BEGIN { warnings::unimport 'redundant' if $^V gt v5.21.1 }
-    $tb->ok(!$thing_meta->does_role($_), sprintf($message, $_))
+    $tb->ok(!$thing_meta->$_does($_), sprintf($message, $_))
         for @$roles;
 
     return;
@@ -150,12 +167,12 @@ sub has_attribute_ok ($$;$) {
 }
 
 
-sub has_method_ok {
+sub has_method_ok ($@) {
     my ($thing, @methods) = @_;
 
     ### $thing
     my $meta = find_meta($thing);
-    my $name = $meta->name;
+    my $name = _thing_name($thing, $meta);
 
     ### @methods
     $tb->ok(!!$meta->has_method($_), "$name has method $_")
@@ -165,9 +182,9 @@ sub has_method_ok {
 }
 
 
-sub role_wraps_around_method_ok { unshift @_, 'around'; goto \&_role_wraps }
-sub role_wraps_before_method_ok { unshift @_, 'before'; goto \&_role_wraps }
-sub role_wraps_after_method_ok  { unshift @_, 'after';  goto \&_role_wraps }
+sub role_wraps_around_method_ok ($@) { unshift @_, 'around'; goto \&_role_wraps }
+sub role_wraps_before_method_ok ($@) { unshift @_, 'before'; goto \&_role_wraps }
+sub role_wraps_after_method_ok  ($@) { unshift @_, 'after';  goto \&_role_wraps }
 
 sub _role_wraps {
     my ($style, $thing, @methods) = @_;
@@ -186,12 +203,12 @@ sub _role_wraps {
 }
 
 
-sub requires_method_ok {
+sub requires_method_ok ($@) {
     my ($thing, @methods) = @_;
 
     ### $thing
     my $meta = find_meta($thing);
-    my $name = $meta->name;
+    my $name = _thing_name($thing, $meta);
 
     ### @methods
     $tb->ok(!!$meta->requires_method($_), "$name requires method $_")
@@ -201,7 +218,7 @@ sub requires_method_ok {
 }
 
 
-sub is_immutable_ok($;$) {
+sub is_immutable_ok ($;$) {
     my ($thing, $message) = @_;
 
     ### $thing
@@ -211,7 +228,7 @@ sub is_immutable_ok($;$) {
     return $tb->ok($meta->is_immutable, $message);
 }
 
-sub is_not_immutable_ok($;$) {
+sub is_not_immutable_ok ($;$) {
     my ($thing, $message) = @_;
 
     ### $thing
@@ -223,11 +240,11 @@ sub is_not_immutable_ok($;$) {
 
 
 # NOTE: deprecate at some point late 2015
-sub is_role  { goto \&is_role_ok  }
-sub is_class { goto \&is_class_ok }
+sub is_role  ($) { goto \&is_role_ok  }
+sub is_class ($) { goto \&is_class_ok }
 
-sub is_role_ok  { unshift @_, 'Role';  goto \&_is_moosey_ok }
-sub is_class_ok { unshift @_, 'Class'; goto \&_is_moosey_ok }
+sub is_role_ok  ($) { unshift @_, 'Role';  goto \&_is_moosey_ok }
+sub is_class_ok ($) { unshift @_, 'Class'; goto \&_is_moosey_ok }
 
 sub _is_moosey_ok {
     my ($type, $thing) =  @_;
@@ -238,15 +255,27 @@ sub _is_moosey_ok {
     $tb->ok(!!$meta, "$thing_name has a metaclass");
     return unless !!$meta;
 
-    return $tb->ok($meta->isa("Moose::Meta::$type"), "$thing_name is a Moose " . lc $type);
+    my $is_moosey = $meta->isa("Moose::Meta::$type");
+
+    # special check for class -- this will happen when, say, you're validating
+    # an attribute and it's a bog standard Moose::Meta::Attribute: strictly
+    # speaking its metaclass is Class::MOPish, but really,
+    # a Moose::Meta::Attribute is a Moose class.  Or arguably so.  Certainly
+    # in the context of what we're asking about here.  Better approaches to
+    # this welcomed as pull requests :)
+
+    $is_moosey ||= ($meta->name || q{}) =~ /^Moose::Meta::/
+        if $type eq 'Class';
+
+    return $tb->ok($is_moosey, "$thing_name is a Moose " . lc $type);
 }
 
 
 # NOTE: deprecate at some point late 2015
-sub is_anon     { goto \&is_anon_ok     }
-sub is_not_anon { goto \&is_not_anon_ok }
+sub is_anon     ($) { goto \&is_anon_ok     }
+sub is_not_anon ($) { goto \&is_not_anon_ok }
 
-sub is_anon_ok {
+sub is_anon_ok ($) {
     my ($thing, $message) = @_;
 
     my $thing_meta = find_meta($thing);
@@ -255,7 +284,7 @@ sub is_anon_ok {
     return $tb->ok(!!$thing_meta->is_anon, $message);
 }
 
-sub is_not_anon_ok {
+sub is_not_anon_ok ($) {
     my ($thing, $message) = @_;
 
     my $thing_meta = find_meta($thing);
@@ -265,9 +294,7 @@ sub is_not_anon_ok {
 }
 
 
-sub known_sugar() { qw{ has around augment inner before after blessed confess } }
-
-sub check_sugar_removed_ok($) {
+sub check_sugar_removed_ok ($) {
     my $t = shift @_;
 
     # check some (not all) Moose sugar to make sure it has been cleared
@@ -277,7 +304,7 @@ sub check_sugar_removed_ok($) {
 }
 
 
-sub check_sugar_ok($) {
+sub check_sugar_ok ($) {
     my $t = shift @_;
 
     # check some (not all) Moose sugar to make sure it has been cleared
@@ -287,10 +314,32 @@ sub check_sugar_ok($) {
 }
 
 
+sub does_metaroles_ok($$)     { push @_, \&does_ok;     goto &_does_metaroles }
+sub does_not_metaroles_ok($$) { push @_, \&does_not_ok; goto &_does_metaroles }
 
-sub validate_thing { _validate_subtest_wrapper(\&_validate_thing_guts, @_) }
-sub validate_class { _validate_subtest_wrapper(\&_validate_class_guts, @_) }
-sub validate_role  { _validate_subtest_wrapper(\&_validate_role_guts,  @_) }
+sub _does_metaroles {
+    my ($thing, $metaroles, $test_func) = @_;
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $meta = find_meta($thing);
+    my $name = _thing_name($thing, $meta);
+
+    for my $mop (sort keys %$metaroles) {
+
+        my $mop_metaclass = get_mop_metaclass_for $mop => $meta;
+
+        local $THING_NAME = "${name}'s $mop metaclass $mop_metaclass";
+        $test_func->($mop_metaclass => $metaroles->{$mop});
+    }
+
+    return;
+}
+
+
+sub validate_thing ($@) { _validate_subtest_wrapper(\&_validate_thing_guts, @_) }
+sub validate_class ($@) { _validate_subtest_wrapper(\&_validate_class_guts, @_) }
+sub validate_role  ($@) { _validate_subtest_wrapper(\&_validate_role_guts,  @_) }
 
 sub _validate_subtest_wrapper {
     my ($func, $thing, %args) = @_;
@@ -360,7 +409,7 @@ sub _validate_class_guts {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unless is_class_ok $class;
 
-    my $name = ref $class || $class;
+    my $name = _thing_name($class);
     do { ok($class->isa($_), "$name isa $_") for @{$args{isa}} }
         if exists $args{isa};
 
@@ -370,6 +419,9 @@ sub _validate_class_guts {
     do { is_not_immutable_ok $class }
         if exists $args{immutable} && !$args{immutable};
 
+    do { does_metaroles_ok $class => $args{class_metaroles} }
+        if exists $args{class_metaroles};
+
     return validate_thing $class => %args;
 }
 
@@ -417,6 +469,7 @@ sub _validate_role_guts {
     $args{does} = [ $role, @{ $args{does} || [] } ];
 
     # aaaand a subtest wrapper to make it easier to read...
+    local $THING_NAME = _thing_name($role) . q{'s composed class};
     return validate_class $anon->name => (
         -subtest => 'role composed into ' . $anon->name,
         %args,
@@ -424,8 +477,8 @@ sub _validate_role_guts {
 }
 
 
-sub _validate_attribute { _validate_subtest_wrapper(\&__validate_attribute_guts,                 @_) }
-sub validate_attribute  { _validate_subtest_wrapper( \&_validate_attribute_guts, [shift, shift], @_) }
+sub _validate_attribute       { _validate_subtest_wrapper(\&__validate_attribute_guts,                 @_) }
+sub validate_attribute ($$@)  { _validate_subtest_wrapper( \&_validate_attribute_guts, [shift, shift], @_) }
 
 sub _validate_attribute_guts {
     my ($thingname, %opts) = @_;
@@ -435,6 +488,7 @@ sub _validate_attribute_guts {
     return unless has_attribute_ok($thing, $name);
     my $att = _find_attribute($thing => $name);
 
+    local $THING_NAME = _thing_name($thing) . "'s attribute $name";
     return _validate_attribute($att, %opts);
 }
 
@@ -453,20 +507,26 @@ sub __validate_attribute_guts {
         if $thing_opts{does};
 
     ### %thing_opts
-    validate_class $att => %thing_opts
-        if keys %thing_opts;
+    {
+        # If $THING_NAME is set, we're processing an attribute metaclass via
+        # _validate_attribute_guts() or _validate_thing_guts()
+        local $THING_NAME = "${THING_NAME}'s metaclass"
+            if !!$THING_NAME;
+        validate_class $att => %thing_opts
+            if keys %thing_opts;
+    }
 
     return _attribute_options_ok($att, %opts);
 }
 
-sub attribute_options_ok {
+sub attribute_options_ok ($$@) {
     my ($thing, $name, %opts) = @_;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unless has_attribute_ok($thing, $name);
     my $att = _find_attribute($thing => $name);
 
-    return _attribute_options_ok($att, %opts);
+    return _validate_subtest_wrapper(\&_attribute_options_ok => ($att, %opts));
 }
 
 sub _attribute_options_ok {
@@ -484,14 +544,20 @@ sub _role_attribute_options_ok {
     my $name                    = $att->name;
     my $thing_name              = _thing_name($name, $att);
 
-    # this much works, at least
-    if (exists $opts{coerce}) {
+    exists $opts{required} and delete $opts{required}
+        ? ok($att->is_required,  "$thing_name is required")
+        : ok(!$att->is_required, "$thing_name is not required")
+        ;
 
-        delete $opts{coerce}
-            ? ok( $att->should_coerce, "$thing_name should coerce")
-            : ok(!$att->should_coerce, "$thing_name should not coerce")
-            ;
-    }
+    exists $opts{lazy} and delete $opts{lazy}
+        ? ok($att->is_lazy,  "$thing_name is lazy")
+        : ok(!$att->is_lazy, "$thing_name is not lazy")
+        ;
+
+    exists $opts{coerce} and delete $opts{coerce}
+        ? ok( $att->should_coerce, "$thing_name should coerce")
+        : ok(!$att->should_coerce, "$thing_name should not coerce")
+        ;
 
     ### for now, skip role attributes: blessed $att
     return $tb->skip('cannot yet test role attribute layouts')
@@ -501,10 +567,6 @@ sub _role_attribute_options_ok {
 sub _class_attribute_options_ok {
     my ($att, %opts) = @_;
 
-    ### for now, skip role attributes: blessed $att
-    return $tb->skip('cannot yet test role attribute layouts')
-        if $att->isa('Moose::Meta::Role::Attribute');
-
     my @check_opts =
         qw{ reader writer accessor predicate default builder clearer };
     my @unhandled_opts = qw{ isa does handles traits };
@@ -537,13 +599,10 @@ sub _class_attribute_options_ok {
         is($att->$property, $value, "$thing_name option $property correct")
     };
 
-    if (my $is_required = delete $opts{required}) {
-
-        $is_required
-            ? ok($att->is_required,  "$thing_name is required")
-            : ok(!$att->is_required, "$thing_name is not required")
-            ;
-    }
+    exists $opts{required} and delete $opts{required}
+        ? ok($att->is_required,  "$thing_name is required")
+        : ok(!$att->is_required, "$thing_name is not required")
+        ;
 
     $check->($_) for grep { any(@check_opts) eq $_ } sort keys %opts;
 
@@ -559,21 +618,15 @@ sub _class_attribute_options_ok {
         delete $opts{init_arg};
     }
 
-    if (exists $opts{lazy}) {
+    exists $opts{lazy} and delete $opts{lazy}
+        ? ok($att->is_lazy,  "$thing_name is lazy")
+        : ok(!$att->is_lazy, "$thing_name is not lazy")
+        ;
 
-        delete $opts{lazy}
-            ? ok($att->is_lazy,  "$thing_name is lazy")
-            : ok(!$att->is_lazy, "$thing_name is not lazy")
-            ;
-    }
-
-    if (exists $opts{coerce}) {
-
-        delete $opts{coerce}
-            ? ok( $att->should_coerce, "$thing_name should coerce")
-            : ok(!$att->should_coerce, "$thing_name should not coerce")
-            ;
-    }
+    exists $opts{coerce} and delete $opts{coerce}
+        ? ok( $att->should_coerce, "$thing_name should coerce")
+        : ok(!$att->should_coerce, "$thing_name should not coerce")
+        ;
 
     for my $opt (sort keys %opts) {
 
@@ -599,7 +652,7 @@ __END__
 
 =encoding UTF-8
 
-=for :stopwords Chris Weyl Chad Etheridge Granum Karen subtest
+=for :stopwords Chris Weyl Chad Etheridge Granum Karen subtest MOPs metaroles
 
 =for :stopwords Wishlist flattr flattr'ed gittip gittip'ed
 
@@ -609,7 +662,7 @@ Test::Moose::More - More tools for testing Moose packages
 
 =head1 VERSION
 
-This document describes version 0.038 of Test::Moose::More - released June 22, 2016 as part of Test-Moose-More.
+This document describes version 0.042 of Test::Moose::More - released February 13, 2017 as part of Test-Moose-More.
 
 =head1 SYNOPSIS
 
@@ -627,11 +680,30 @@ This package contains a number of additional tests that can be employed
 against Moose classes/roles.  It is intended to replace L<Test::Moose> in your
 tests, and re-exports any tests that it has and we do not, yet.
 
+=head2 Export Groups
+
+By default, this package exports all test functions.  You can be more
+selective, however, and there are a number of export groups (aside from the
+default ':all') to help you achieve those dreams!
+
+=over 4
+
+=item :all
+
+All exportable functions.
+
+=item :validate
+
+L</validate_attribute>, L</validate_class>, L</validate_role>, L</validate_thing>
+
+=back
+
 =head1 FUNCTIONS
 
-=head2 known_sugar
+=head2 does_not_metaroles_ok $thing => { $mop => [ @traits ], ... };
 
-Returns a list of all the known standard Moose sugar (has, extends, etc).
+As with L</does_metaroles_ok>, but test that the metaroles are not consumed, a
+la L</does_not_ok>.
 
 =head1 TEST FUNCTIONS
 
@@ -720,6 +792,70 @@ given package.
 
 Checks and makes sure a class/etc can still do all the standard Moose sugar.
 
+=head2 does_metaroles_ok $thing => { $mop => [ @traits ], ... };
+
+Validate the metaclasses associated with a class/role metaclass.
+
+e.g., if I wanted to validate that the attribute trait for
+L<MooseX::AttributeShortcuts> is actually applied, I could do this:
+
+    { package TestClass; use Moose; use MooseX::AttributeShortcuts; }
+    use Test::Moose::More;
+    use Test::More;
+
+    does_metaroles_ok TestClass => {
+       attribute => ['MooseX::AttributeShortcuts::Trait::Attribute'],
+    };
+    done_testing;
+
+This function will accept either class or role metaclasses for $thing.
+
+The MOPs available for classes (L<Moose::Meta::Class>) are:
+
+=over 4
+
+=item class
+
+=item attribute
+
+=item method
+
+=item wrapped_method
+
+=item instance
+
+=item constructor
+
+=item destructor
+
+=back
+
+The MOPs available for roles (L<Moose::Meta::Role>) are:
+
+=over 4
+
+=item role
+
+=item attribute
+
+=item method
+
+=item required_method
+
+=item wrapped_method
+
+=item conflicting_method
+
+=item application_to_class
+
+=item application_to_role
+
+=item application_to_instance
+
+=item applied_attribute
+
+=back
+
 =head2 validate_thing
 
 Runs a bunch of tests against the given C<$thing>, as defined:
@@ -841,14 +977,14 @@ e.g.:
     ok 5 - TestRole does not do TestRole::Two
     ok 6 - TestRole has method method1
     ok 7 - TestRole has an attribute named bar
-        # Subtest: role composed into Moose::Meta::Class::__ANON__::SERIAL::1
-        ok 1 - Moose::Meta::Class::__ANON__::SERIAL::1 has a metaclass
-        ok 2 - Moose::Meta::Class::__ANON__::SERIAL::1 is a Moose class
-        ok 3 - Moose::Meta::Class::__ANON__::SERIAL::1 does TestRole
-        ok 4 - Moose::Meta::Class::__ANON__::SERIAL::1 does not do TestRole::Two
-        ok 5 - Moose::Meta::Class::__ANON__::SERIAL::1 has method method1
-        ok 6 - Moose::Meta::Class::__ANON__::SERIAL::1 has method blargh
-        ok 7 - Moose::Meta::Class::__ANON__::SERIAL::1 has an attribute named bar
+    # Subtest: role composed into Moose::Meta::Class::__ANON__::SERIAL::1
+        ok 1 - TestRole's composed class has a metaclass
+        ok 2 - TestRole's composed class is a Moose class
+        ok 3 - TestRole's composed class does TestRole
+        ok 4 - TestRole's composed class does not do TestRole::Two
+        ok 5 - TestRole's composed class has method method1
+        ok 6 - TestRole's composed class has method blargh
+        ok 7 - TestRole's composed class has an attribute named bar
         1..7
     ok 8 - role composed into Moose::Meta::Class::__ANON__::SERIAL::1
     1..8
@@ -903,7 +1039,6 @@ additional class-specific tests.
 
         attributes => [ ... ],
         methods    => [ ... ],
-        isa        => [ ... ],
 
         # ensures sugar is/is-not present
         sugar      => 0,
@@ -1001,6 +1136,14 @@ Note that some of these options will skip if used against attributes defined in
 
 =item *
 
+-subtest => 'subtest name...'
+
+If set, all tests run (save the first, "does this thing even have this
+attribute?" test) will be wrapped in a subtest, the name of which will be
+whatever C<-subtest> is set to.
+
+=item *
+
 is => ro|rw
 
 Tests for reader/writer options set as one would expect.
@@ -1066,7 +1209,7 @@ L<Test::Moose>
 =head1 BUGS
 
 Please report any bugs or feature requests on the bugtracker website
-https://github.com/RsrchBoy/Test-Moose-More/issues
+L<https://github.com/RsrchBoy/Test-Moose-More/issues>
 
 When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
